@@ -6,23 +6,27 @@ License: MIT
 Version: 1.0.0
 Email: tyep@cs.stanford.edu
 """
+from __future__ import annotations
+
 from enum import Enum, unique
+from typing import Any, Dict, Iterator, Tuple
 
-from .karel_definitions import Direction
+from .karel_world import Direction, KarelWorld
 
-CHAR_WIDTH = 6
+CHAR_WIDTH = 5
 HORIZONTAL, VERTICAL = "─", "│"
 SPACING = 10
+BEEPER_COORDS = Dict[Tuple[int, int], int]
 
 
-class Tile:  # pylint: disable=too-few-public-methods
-    def __init__(self, value="·"):
+class Tile:
+    def __init__(self, value: str = "·") -> None:
         self.value = value
-        self.walls = []
+        self.walls: list[Direction] = []
         self.beepers = 0
         self.color = ""
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         result = ""
         if self.value == "K" and self.beepers > 0:
             result = "<K>"
@@ -32,11 +36,11 @@ class Tile:  # pylint: disable=too-few-public-methods
             result = self.color[:3]
         else:
             result = self.value
-        return result.center(5)
+        return result.center(CHAR_WIDTH)
 
 
 class AsciiKarelWorld:
-    def __init__(self, world, karel_street, karel_avenue):
+    def __init__(self, world: KarelWorld, karel_street: int, karel_avenue: int) -> None:
         num_sts, num_aves = world.num_streets, world.num_avenues
         # Initialize Tiles
         self.world_arr = [[Tile() for _ in range(num_aves)] for _ in range(num_sts)]
@@ -60,8 +64,8 @@ class AsciiKarelWorld:
         self.num_streets = num_sts
         self.num_avenues = num_aves
 
-    def __repr__(self):
-        avenue_widths = HORIZONTAL * (CHAR_WIDTH * self.num_avenues + 1)
+    def __repr__(self) -> str:
+        avenue_widths = HORIZONTAL * ((CHAR_WIDTH + 1) * self.num_avenues + 1)
         result = f"┌{avenue_widths}┐\n"
         for r in range(self.num_streets):
             next_line = VERTICAL
@@ -82,13 +86,13 @@ class AsciiKarelWorld:
                 result += f"{next_line} {VERTICAL}\n"
         return result
 
-    def tile_has_wall(self, r, c, direction):
+    def tile_has_wall(self, r: int, c: int, direction: Direction) -> bool:
         if 0 <= r < self.num_streets and 0 <= c < self.num_avenues:
             tile = self.world_arr[r][c]
             return direction in tile.walls
         return False
 
-    def tile_pair_has_wall(self, r, c, direction):
+    def tile_pair_has_wall(self, r: int, c: int, direction: Direction) -> bool:
         """
         Checks whether the tile at r, c should have a wall by
         checking itself and its neighbors.
@@ -111,8 +115,8 @@ class AsciiKarelWorld:
             )
         raise ValueError("Direction is invalid.")
 
-    def get_next_line(self, r, c, next_block_start):
-        """ Given a tile, figures out the lower line of the ASCII art. """
+    def get_next_line(self, r: int, c: int, next_block_start: str) -> tuple[str, str]:
+        """Given a tile, figures out the lower line of the ASCII art."""
 
         if self.tile_pair_has_wall(r, c, Direction.SOUTH):
             if (
@@ -131,7 +135,7 @@ class AsciiKarelWorld:
                 next_block_start = "┌"
             elif self.tile_pair_has_wall(r, c, Direction.WEST):
                 next_block_start = "└"
-            next_line = next_block_start + HORIZONTAL * (CHAR_WIDTH - 1)
+            next_line = next_block_start + HORIZONTAL * CHAR_WIDTH
             next_block_start = HORIZONTAL
         else:
             if (
@@ -153,7 +157,7 @@ class AsciiKarelWorld:
             ) and self.tile_pair_has_wall(r + 1, c, Direction.WEST):
                 next_block_start = VERTICAL
 
-            next_line = next_block_start + " " * (CHAR_WIDTH - 1)
+            next_line = next_block_start + " " * CHAR_WIDTH
             next_block_start = " "
         return next_line, next_block_start
 
@@ -172,14 +176,16 @@ class Color(Enum):
     END = "\033[0m"
 
 
-def compare_output(first, second):
-    """ Compares Karel Output and gets the results. """
+def compare_output(first: Any, second: Any) -> str:
+    """Compares Karel Output and gets the results."""
 
-    def create_two_column_string(col1, col2):
-        """ col1 and col2 are Lists. """
+    def create_two_column_string(col1: list[str], col2: list[str]) -> Iterator[str]:
+        """col1 and col2 are Lists."""
         return map(lambda x: f"{x[0]}{' ' * SPACING}{x[1]}", zip(col1, col2))
 
-    def symmetric_difference(a, b):
+    def symmetric_difference(
+        a: BEEPER_COORDS, b: BEEPER_COORDS
+    ) -> tuple[BEEPER_COORDS, BEEPER_COORDS]:
         extra_a, extra_b = {}, {}
         for k in a:
             if k not in b:
@@ -204,7 +210,7 @@ def compare_output(first, second):
 
     result = (
         f"\n\n{fancy_arrows} "
-        f"{Color.YELLOW.value}{first.world.world_name}{Color.END.value}"
+        f"{Color.YELLOW.value}{first.world.world_file}{Color.END.value}"
         f"\n{header1}{text_spacing}{header2}\n{output}\n"
     )
 
@@ -225,8 +231,3 @@ def compare_output(first, second):
             f"Expected: {extra_b}\n\n"
         )
     return result
-
-
-def karel_ascii(world, karel_street, karel_avenue):
-    """ Creates a Karel World in ASCII Art! """
-    return str(AsciiKarelWorld(world, karel_street, karel_avenue))
