@@ -11,14 +11,15 @@ License: MIT
 Version: 1.0.0
 Email: nbowman@stanford.edu
 Date of Creation: 10/1/2019
-Last Modified: 3/31/2020
 """
+from __future__ import annotations
 
 import cmath
 import math
 import tkinter as tk
 
-from .karel_definitions import DEFAULT_ICON, Direction, Wall
+from .karel_program import KarelProgram
+from .karel_world import Direction, KarelWorld, Wall
 
 DIRECTION_TO_RADIANS = {
     Direction.EAST: 0,
@@ -26,6 +27,13 @@ DIRECTION_TO_RADIANS = {
     Direction.WEST: math.pi,
     Direction.NORTH: 3 * math.pi / 2,
 }
+
+# Karel Application + World Editor
+DEFAULT_ICON = "karel"
+PAD_X = 75
+PAD_Y = 10
+LIGHT_GREY = "#e5e5e5"
+
 WALL_DETECTION_THRESHOLD = 0.1
 BORDER_OFFSET = 17
 LABEL_OFFSET = 7
@@ -57,7 +65,15 @@ SIMPLE_KAREL_WIDTH = 0.8
 
 
 class KarelCanvas(tk.Canvas):
-    def __init__(self, width, height, master, world=None, karel=None, bg="white"):
+    def __init__(
+        self,
+        width: int,
+        height: int,
+        master: tk.Misc,
+        world: KarelWorld,
+        karel: KarelProgram,
+        bg: str = "white",
+    ) -> None:
         super().__init__(master, width=width, height=height, bg=bg)
         self.world = world
         self.karel = karel
@@ -66,56 +82,61 @@ class KarelCanvas(tk.Canvas):
         self.draw_karel()
 
     @staticmethod
-    def rotate_points(center, points, direction):
+    def rotate_points(
+        center: tuple[float, float], points: list[float], direction: float
+    ) -> None:
         """
         Rotation logic derived from http://effbot.org/zone/tkinter-complex-canvas.htm
         """
         cangle = cmath.exp(direction * 1j)
-        center = complex(center[0], center[1])
+        ccenter = complex(center[0], center[1])
         for i in range(0, len(points), 2):
             x, y = points[i], points[i + 1]
-            v = cangle * (complex(x, y) - center) + center
+            v = cangle * (complex(x, y) - ccenter) + ccenter
             points[i], points[i + 1] = v.real, v.imag
 
-    def create_polygon(self, points, fill="black", outline="black", tag="karel"):
+    def create_default_polygon(
+        self,
+        points: list[float],
+        fill: str = "black",
+        outline: str = "black",
+        tags: str = "karel",
+    ) -> None:
         super().create_polygon(
-            points, fill=fill, outline=outline, width=KAREL_LINE_WIDTH, tag=tag
+            *points, fill=fill, outline=outline, width=KAREL_LINE_WIDTH, tags=tags
         )
 
-    def set_icon(self, icon):
-        self.icon = icon
-
-    def redraw_all(self):
+    def redraw_all(self) -> None:
         self.delete("all")
         self.draw_world()
         self.draw_karel()
         self.update()
 
-    def redraw_karel(self, update=True):
+    def redraw_karel(self, update: bool = True) -> None:
         self.delete("karel")
         self.draw_karel()
         if update:
             self.update()
 
-    def redraw_beepers(self, update=True):
+    def redraw_beepers(self, update: bool = True) -> None:
         self.delete("beeper")
         self.draw_all_beepers()
         if update:
             self.update()
 
-    def redraw_corners(self, update=True):
+    def redraw_corners(self, update: bool = True) -> None:
         self.delete("corner")
         self.draw_corners()
         if update:
             self.update()
 
-    def redraw_walls(self, update=True):
+    def redraw_walls(self, update: bool = True) -> None:
         self.delete("wall")
         self.draw_all_walls()
         if update:
             self.update()
 
-    def draw_world(self):
+    def draw_world(self) -> None:
         self.init_geometry_values()
         self.draw_bounding_rectangle()
         self.label_axes()
@@ -123,7 +144,7 @@ class KarelCanvas(tk.Canvas):
         self.draw_all_beepers()
         self.draw_all_walls()
 
-    def init_geometry_values(self):
+    def init_geometry_values(self) -> None:
         self.update()
 
         # Calculate the maximum possible cell size in both directions
@@ -147,7 +168,7 @@ class KarelCanvas(tk.Canvas):
         self.right_x = self.left_x + self.boundary_width
         self.bottom_y = self.top_y + self.boundary_height
 
-    def draw_bounding_rectangle(self):
+    def draw_bounding_rectangle(self) -> None:
         # Draw the external bounding lines of Karel's world
         self.create_line(
             self.left_x, self.top_y, self.right_x, self.top_y, width=LINE_WIDTH
@@ -162,7 +183,7 @@ class KarelCanvas(tk.Canvas):
             self.left_x, self.bottom_y, self.right_x, self.bottom_y, width=LINE_WIDTH
         )
 
-    def label_axes(self):
+    def label_axes(self) -> None:
         # Label the avenue axes
         for avenue in range(1, self.world.num_avenues + 1):
             label_x = self.calculate_corner_x(avenue)
@@ -175,7 +196,7 @@ class KarelCanvas(tk.Canvas):
             label_y = self.calculate_corner_y(street)
             self.create_text(label_x, label_y, text=str(street), font="Arial 10")
 
-    def draw_corners(self):
+    def draw_corners(self) -> None:
         # Draw all corner markers in the world
         for avenue in range(1, self.world.num_avenues + 1):
             for street in range(1, self.world.num_streets + 1):
@@ -188,14 +209,14 @@ class KarelCanvas(tk.Canvas):
                         corner_y - CORNER_SIZE,
                         corner_x,
                         corner_y + CORNER_SIZE,
-                        tag="corner",
+                        tags="corner",
                     )
                     self.create_line(
                         corner_x - CORNER_SIZE,
                         corner_y,
                         corner_x + CORNER_SIZE,
                         corner_y,
-                        tag="corner",
+                        tags="corner",
                     )
                 else:
                     self.create_rectangle(
@@ -204,15 +225,15 @@ class KarelCanvas(tk.Canvas):
                         corner_x + self.cell_size / 2,
                         corner_y + self.cell_size / 2,
                         fill=color,
-                        tag="corner",
+                        tags="corner",
                         outline="",
                     )
 
-    def draw_all_beepers(self):
+    def draw_all_beepers(self) -> None:
         for location, count in self.world.beepers.items():
             self.draw_beeper(location, count)
 
-    def draw_beeper(self, location, count):
+    def draw_beeper(self, location: tuple[int, int], count: int) -> None:
         # handle case where defaultdict returns 0 count by not drawing beepers
         if count == 0:
             return
@@ -231,18 +252,18 @@ class KarelCanvas(tk.Canvas):
             corner_x - beeper_radius,
             corner_y,
         ]
-        self.create_polygon(points, fill="light grey", tag="beeper")
+        self.create_default_polygon(points, fill="light grey", tags="beeper")
 
         if count > 1:
             self.create_text(
-                corner_x, corner_y, text=str(count), font="Arial 12", tag="beeper"
+                corner_x, corner_y, text=str(count), font="Arial 12", tags="beeper"
             )
 
-    def draw_all_walls(self):
+    def draw_all_walls(self) -> None:
         for wall in self.world.walls:
             self.draw_wall(wall)
 
-    def draw_wall(self, wall):
+    def draw_wall(self, wall: Wall) -> None:
         avenue, street, direction = wall.avenue, wall.street, wall.direction
         corner_x = self.calculate_corner_x(avenue)
         corner_y = self.calculate_corner_y(street)
@@ -254,7 +275,7 @@ class KarelCanvas(tk.Canvas):
                 corner_x + self.cell_size / 2,
                 corner_y - self.cell_size / 2,
                 width=LINE_WIDTH,
-                tag="wall",
+                tags="wall",
             )
         if direction == Direction.SOUTH:
             self.create_line(
@@ -263,7 +284,7 @@ class KarelCanvas(tk.Canvas):
                 corner_x + self.cell_size / 2,
                 corner_y + self.cell_size / 2,
                 width=LINE_WIDTH,
-                tag="wall",
+                tags="wall",
             )
         if direction == Direction.EAST:
             self.create_line(
@@ -272,7 +293,7 @@ class KarelCanvas(tk.Canvas):
                 corner_x + self.cell_size / 2,
                 corner_y + self.cell_size / 2,
                 width=LINE_WIDTH,
-                tag="wall",
+                tags="wall",
             )
         if direction == Direction.WEST:
             self.create_line(
@@ -281,10 +302,10 @@ class KarelCanvas(tk.Canvas):
                 corner_x - self.cell_size / 2,
                 corner_y + self.cell_size / 2,
                 width=LINE_WIDTH,
-                tag="wall",
+                tags="wall",
             )
 
-    def draw_karel(self):
+    def draw_karel(self) -> None:
         corner_x = self.calculate_corner_x(self.karel.avenue)
         corner_y = self.calculate_corner_y(self.karel.street)
         center = (corner_x, corner_y)
@@ -316,7 +337,9 @@ class KarelCanvas(tk.Canvas):
                 center, DIRECTION_TO_RADIANS[self.karel.direction]
             )
 
-    def generate_external_karel_points(self, x, y, center, direction):
+    def generate_external_karel_points(
+        self, x: float, y: float, center: tuple[float, float], direction: float
+    ) -> list[float]:
         outer_points = []
 
         # Top-left point (referred to as origin) of Karel's body
@@ -347,7 +370,9 @@ class KarelCanvas(tk.Canvas):
 
         return outer_points
 
-    def generate_internal_karel_points(self, x, y, center, direction):
+    def generate_internal_karel_points(
+        self, x: float, y: float, center: tuple[float, float], direction: float
+    ) -> list[float]:
 
         # Calculate dimensions and location of Karel's inner eye
         inner_x = x + self.cell_size * KAREL_INNER_OFFSET
@@ -372,7 +397,9 @@ class KarelCanvas(tk.Canvas):
 
         return inner_points
 
-    def draw_karel_body(self, x, y, center, direction):
+    def draw_karel_body(
+        self, x: float, y: float, center: tuple[float, float], direction: float
+    ) -> None:
         outer_points = self.generate_external_karel_points(x, y, center, direction)
         inner_points = self.generate_internal_karel_points(x, y, center, direction)
 
@@ -381,11 +408,11 @@ class KarelCanvas(tk.Canvas):
         entire_body_points = outer_points + inner_points
 
         # First draw the filled non-convex polygon
-        self.create_polygon(entire_body_points, fill="white", outline="")
+        self.create_default_polygon(entire_body_points, fill="white", outline="")
 
         # Then draw the transparent exterior edges of Karel's body
-        self.create_polygon(outer_points, fill="")
-        self.create_polygon(inner_points, fill="")
+        self.create_default_polygon(outer_points, fill="")
+        self.create_default_polygon(inner_points, fill="")
 
         # Define dimensions and location of Karel's mouth
         # karel_height = self.cell_size * KAREL_HEIGHT
@@ -405,9 +432,11 @@ class KarelCanvas(tk.Canvas):
             mouth_y,
         ]
         self.rotate_points(center, mouth_points, direction)
-        self.create_polygon(mouth_points, fill="white")
+        self.create_default_polygon(mouth_points, fill="white")
 
-    def draw_karel_legs(self, x, y, center, direction):
+    def draw_karel_legs(
+        self, x: float, y: float, center: tuple[float, float], direction: float
+    ) -> None:
         leg_length = self.cell_size * KAREL_LEG_LENGTH
         foot_length = self.cell_size * KAREL_FOOT_LENGTH
         leg_foot_width = self.cell_size * KAREL_LEG_FOOT_WIDTH
@@ -429,7 +458,7 @@ class KarelCanvas(tk.Canvas):
         points += [x, y + vertical_offset]
 
         self.rotate_points(center, points, direction)
-        self.create_polygon(points)
+        self.create_default_polygon(points)
 
         # Reset point of reference to be bottom left rather than top_left
         y += self.cell_size * KAREL_HEIGHT
@@ -448,9 +477,11 @@ class KarelCanvas(tk.Canvas):
         points += [x + horizontal_offset, y]
 
         self.rotate_points(center, points, direction)
-        self.create_polygon(points)
+        self.create_default_polygon(points)
 
-    def draw_simple_karel_icon(self, center, direction):
+    def draw_simple_karel_icon(
+        self, center: tuple[float, float], direction: float
+    ) -> None:
         simple_karel_width = self.cell_size * SIMPLE_KAREL_WIDTH
         simple_karel_height = self.cell_size * SIMPLE_KAREL_HEIGHT
         center_x, center_y = center
@@ -471,32 +502,34 @@ class KarelCanvas(tk.Canvas):
             center_y - simple_karel_height / 2,
         ]
         self.rotate_points(center, points, direction)
-        self.create_polygon(points, fill="white")
+        self.create_default_polygon(points, fill="white")
 
-    def calculate_corner_x(self, avenue):
+    def calculate_corner_x(self, avenue: float) -> float:
         return self.left_x + self.cell_size / 2 + (avenue - 1) * self.cell_size
 
-    def calculate_corner_y(self, street):
+    def calculate_corner_y(self, street: float) -> float:
         return (
             self.top_y
             + self.cell_size / 2
             + (self.world.num_streets - street) * self.cell_size
         )
 
-    def click_in_world(self, x, y):
+    def click_in_world(self, x: float, y: float) -> bool:
         x = x - self.left_x
         y = y - self.top_y
         return 0 <= x < self.boundary_width and 0 <= y < self.boundary_height
 
-    def calculate_location(self, x, y):
+    def calculate_location(self, x: float, y: float) -> tuple[float, float]:
         x = x - self.left_x
         y = y - self.top_y
         return (
-            int(max(x, 0) // self.cell_size) + 1,
-            int(max((self.boundary_height - 1 - y), 0) // self.cell_size) + 1,
+            max(x, 0) // self.cell_size + 1,
+            max((self.boundary_height - 1 - y), 0) // self.cell_size + 1,
         )
 
-    def find_nearest_wall(self, x, y, avenue, street):
+    def find_nearest_wall(
+        self, x: float, y: float, avenue: int, street: int
+    ) -> Wall | None:
         corner_x = self.calculate_corner_x(avenue)
         corner_y = self.calculate_corner_y(street)
         wall_proximity = self.cell_size * WALL_DETECTION_THRESHOLD
